//===========================================================================
//! @file	main.cpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! This is the main entry point for the data set generator program.
//!
//! This program generates a set of random data points to be used in
//! analysing the performance of various data search and mining algorithms.
//!
//! The set of data points created can have as much or as little structure
//! as you like.
//!
//!	In this program all of the spatial structure comes from a very special
//! type of scalar field function (perlin noise) with an additional simple
//! acceptance function. Perlin noise was designed to simulate various natural
//! phenomina).
//!		For a great tutorial on perlin noise see:
//!			http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
//!
//!		For Ken Perlin's hompage see:
//!			http://mrl.nyu.edu/~perlin/
//!
//!		The points are generated as follows:
//!		1. Generate a random candidate point within a hypercube of
//!			dimension desired.
//!		2. Test the point against a fuzzy spatial structure within that
//!			dimensional space and either accept or reject the candidate.
//!		3. Repeat steps one and two until a candidate passes.
//!		4. Repeat steps one through three until the desired number of
//!			points have been acquired.
//!
//!		The fuzzy spatial structure used below is defined by:
//!			- The settings of the noise (seed and frequency).
//!			- Getting the value ( 0.0 to 1.0) of the noise function at
//!				the point.
//!			- How we use the value to either keep or reject the candidate
//!				point using a hardThreshold, fuzzyTheshold, and a fuzzyGamma.
//!
//!		Note: since this a culling based method you should make sure that
//!		your spatial structure (the perlin noise based shape) isn't to thin
//!		or sparse, since this will mean lots of failed candidates and
//!		consequently very long processing times.
//===========================================================================
#include <stdio.h>			// For fprintf and stderr
#include "ndpoint.hpp"
#include "ndspatialsets.hpp"


//---------------------------------------------------------------------------
//! The entry point for the entire program.
//---------------------------------------------------------------------------
int main( int argc, char* argv[] )
{
	const unsigned int	numPointsDesired					= 100;
	const unsigned int	numDimensions						= 6;
	const unsigned int	pointSeed							= 86376948;
	const unsigned int	noiseRandomSeed						= 25877867;
	const float			noiseFrequency						= 1.0f;
	const float			acceptanceHardThreshholdPortion		= 0.45f;
	const float			acceptanceFuzzyThreshholdPortion	= 0.55f;
	const unsigned int	acceptanceRandomSeed				= 91881465;
	
	// Let the user know we've startedto init our genreation system.
	fprintf( stderr, "initializing...\n" );
	
	// Create a basic generator for the candidate points.
	NDPointUniformRandomGenerator candidateGenerator( pointSeed, 0.5f );
	
	// Create a candidate point to fill.
	NDPoint candidatePoint( numDimensions );

	// Create the spatial structure.
	NDSpatialSet* pSpatialSet = 0;// Set to null initially.
	NDSpatialSetNoise	spatialSet(
		numDimensions,
		noiseRandomSeed,
		noiseFrequency,
		acceptanceHardThreshholdPortion,
		acceptanceFuzzyThreshholdPortion,
		acceptanceRandomSeed );
	pSpatialSet = static_cast<NDSpatialSet*>( &spatialSet );

	// Let the user know we've started making the points.
	fprintf( stderr, "processing...\n" );

	// Calculate all the points in the data set.
	for( unsigned int i = 0; i < numPointsDesired; ++i )
	{
		// Generate a valid point.
		// We do this by generating random points until we get one that
		// is a valid member of the spatial set.
		do{
			// Make the candidate point be randomly placed in a unit cube
			// of dimension numDimensions.
			candidateGenerator.GeneratePoint( candidatePoint );

		}while( !pSpatialSet->AcceptMember( candidatePoint ) );

		// Output the point so the user can see it.
		fprintf( stderr, "point: " );
		for( unsigned int i = 0; i < ( numDimensions - 1 ); ++i )
		{
			fprintf( stderr, " %f,", candidatePoint[i] );
		}
		fprintf( stderr, " %f ", candidatePoint[numDimensions-1] );
		fprintf( stderr, "\n" );
	}

	// Let the user know we're done.
	fprintf( stderr, "done!\n" );

	// The program appears to have run fine (since we're here),
	// so let the OS know.
	return 0;
}