//===========================================================================
//! @file	ndnoise.hpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! The original templated version of this code was created by Chris
//! Stephenson (chris@icezip.com) on the 18 December 2001.
//!
//! The original version of this famous function was created by Ken Perlin
//! and is described in the book Texturing & Modeling: A Procedural Approach
//! and on Ken Perlin's web site.
//! The function was designed to simulate various natural phenomina.
//!
//! The "Noise" function was the first function to win as Oscar.
//!
//!		For a great tutorial on perlin noise see:
//!			http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
//!
//!		For Ken Perlin's website see:
//!			http://www.kenperlin.com/talk1/
//!			http://mrl.nyu.edu/~perlin/
//!
//! This is a class to hold a "noisy" scalar field in N dimensions.
//! In this context "noise" means a sin like function where the variation
//! in amplitude is random but fixed for a given input value.
//! The noise generated has the following charcteristics:
//!		- noise produced between -1 and +1
//!		- pseudo-random (same value produced by same vector)
//!		- features roughly the same size (no high or low spatial frequencies)
//!
//! It was purposely un-templated from Chris Stephenson' version to allow for
//! use in dynamic applications where the number of different dimesions needed
//! can be driven by real time processes (such as user input or inside the for
//! loop of a test suite).
//!
//! Also, the code which generated the random gradient pointers differed from
//! the original Perlin code in that vectors greater than 1.0 are supposed to
//! be ignored in Perlin's code. If this is not done, then the generated
//! gradients will tend to cluster around the corners of the unit cube.
//!
//! The member functions Lerp and CubicSpline should probably be in their own
//! maths module, but are included here for brevity.
//===========================================================================
#ifndef NDNOISE_HPP
#define NDNOISE_HPP
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <math.h>		// For sqrtf and floorf
#include "randgen.hpp"
#include "ndpoint.hpp"



//---------------------------------------------------------------------------
//! A class to hold a "noisy" scalar field.
//---------------------------------------------------------------------------
class NDNoise
{
public:

	// Construct the field using the random seed and number of dimesnions
	// specified.
	NDNoise( 	const unsigned int	numDimensions,
				const int			randomSeed )
		: // Initializer list.
		m_numDimensions( numDimensions ),
	  	m_randGen( randomSeed )
	{
		// Set up the random gradients.
		{
			unsigned int i,j;
			m_gradients = new float[num_gradient_vectors * m_numDimensions];

			// Create the gradients.
			for( i = 0; i < buffer_size; ++i )
			{
				// Create a vector inside a unit sphere (r==1.0f).
				float m_s;
				do{
					// Create a gradient in a unit cube (each edge is 2.0f).
					for( j = 0; j < m_numDimensions; ++j )
					{
						m_gradients[i * m_numDimensions + j] =
							static_cast< float >( ( m_randGen() % ( buffer_size + buffer_size ) ) - buffer_size ) /
							static_cast< float >( buffer_size );
					}

					// Get the length of the gradient (used to check if it is in
					// a unit sphere and to normalise it if it is).
					m_s = 0.0f;
					for( j = 0; j < m_numDimensions; ++j )
					{
						m_s += m_gradients[i * m_numDimensions + j] * m_gradients[i * m_numDimensions + j];
					}
				}while( m_s > 1.0f );

				// normalize  the gradient
				m_s = 1.0f / sqrtf( m_s );
				for( j = 0; j < m_numDimensions; ++j )
				{
					m_gradients[i * m_numDimensions + j] *= m_s;
				}
			}

			// copy gradients to the 2nd half of the array.
			for( i = 0; i < buffer_size + buffer_pad; ++i )
			{	
				for( j = 0; j < m_numDimensions; ++j )
				{
					m_gradients[ (buffer_size + i) * m_numDimensions + j] = m_gradients[i * m_numDimensions + j];
				}
			}
		}

		// Set up then randomize the permutation array.
		{
			unsigned int i,j;
			m_pointers = new int[num_gradient_vectors];

			// Init the pointers.
			for( i = 0; i < buffer_size; ++i )
			{
				m_pointers[i] = i;
			}

			// Randomize the pointers.
			unsigned int temp;
			while ( --i )
			{
				temp = m_pointers[i];
				j = ( m_randGen() & buffer_mask );
				m_pointers[i] = m_pointers[j];
				m_pointers[j] = temp;
			}
			
			// Copy the pointers to the 2nd half of the array.
			for( i = 0; i < buffer_size + buffer_pad; ++i )
			{	
				m_pointers[buffer_size + i] = m_pointers[i];
			}
		}

		// Set up the working variables.
		m_ivec =	new int[m_numDimensions];
		m_p =		new int[m_numDimensions];   // pointer offsets
		m_r =		new float[m_numDimensions]; // m_r + m_ivec = point ( to be dotted with the m_gradients )
		m_s =		new float[m_numDimensions]; // interpolation values
	}

	// Destruct the class and free resources.
	~NDNoise()
	{
		delete[] m_pointers;
		delete[] m_gradients;

		delete[] m_ivec;
		delete[] m_p;
		delete[] m_r;
		delete[] m_s;
	}

	// Get the value of the noise functionn at some point
	// in the scalar field.
	inline const float GetValue( const NDPoint& point ) const
	{
		// Set up all the working variables
		for( unsigned int i = 0; i < m_numDimensions; ++i )
		{
			m_ivec[i] = ( int )floorf( point[i] );
			m_p[i] = m_ivec[i] & buffer_mask;
			m_r[i] = point[i] - m_ivec[i];
			m_s[i] = CubicSpline( m_r[i] );
		}

		// Call the recursive helper function.
		const float val = RGet( m_p, m_r, m_s, 0, 0 );

		// Return the value of the scalar field at this point.
		return val;
	}

protected:

	// Linearly interpolate between two floating point values.
	inline const float Lerp( const float t, const float a, const float b ) const
	{
		return ( ( a ) + ( t ) * ( ( b ) - ( a ) ) );
	}

	// Do a cubic interpolation on the range [0,1].
	inline const float CubicSpline( const float t ) const
	{
		return ( t * t * ( 3.0f - 2.0f * t ) );
	}

	// Recursive helper function for GetValue.
	const float RGet( const int m_p[], float m_r[], const float m_s[], const unsigned int cumulative_p, const unsigned int n ) const
	{
		if( n == m_numDimensions )
		{ // Base case: return m_r dot gradient.
			float val = 0.0f;
			for( unsigned int i = 0; i < m_numDimensions; ++i )
			{
				val += m_r[i] * m_gradients[ cumulative_p * m_numDimensions + i ];
			}
			return val;
		}
		else
		{// Recursive case: return interpolation of values.
			float a, b;

			a = RGet( m_p, m_r, m_s, m_pointers[cumulative_p + m_p[n]], n + 1 );

			m_r[n]-=1.0f;
			b = RGet( m_p, m_r, m_s, m_pointers[cumulative_p + m_p[n] + 1], n + 1 );

			m_r[n]+=1.0f;

			return Lerp( m_s[n], a, b );
		}
	}

	// Some class constants
	enum
	{ 
		buffer_size = 0x100,	// buffer size, must be a power of 2 for mask to work
		buffer_mask = 0xff,		// ANDing ( & ) is much faster than MODing ( % )
		buffer_pad = 2,			// a little extra space for simplification of methods
		num_gradient_vectors = buffer_size + buffer_size + buffer_pad
	};

	// Initialisation variables.
	unsigned int			m_numDimensions;
	RandomNumberGenerator	m_randGen;

	// Noise field variables.
	int*					m_pointers;
	float*					m_gradients;

	// Work variables.
	int*					m_ivec;
	int*					m_p;
	float*					m_r;
	float*					m_s;
};


#endif // NDNOISE_HPP