//===========================================================================
//! @file	ndpoint.hpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! This is a class designed to reporesent a single point in an N dimesnional
//! real number space.
//!
//! It is purposely not templated to allow for use in dynamic applications
//! where the number of different dimesions needed can be driven by real time
//! processes (such as user input or inside the for loop of a test suite).
//===========================================================================
#ifndef NDPOINT_HPP
#define NDPOINT_HPP
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "randgen.hpp"


//---------------------------------------------------------------------------
//! A class to hold a point in the specified dimensional space.
//---------------------------------------------------------------------------
struct NDPoint
{
public:

	NDPoint( const unsigned int numDimensions )
		: // Initializer list.
		m_numDimensions( numDimensions )
	{
		m_pData = new float[m_numDimensions];
	}

	virtual ~NDPoint()
	{
		m_numDimensions = 0;
		delete[] m_pData;
	}

	inline const unsigned int GetNumDimensions( void ) const
	{
		return m_numDimensions;
	}

	inline const float& operator[] (unsigned int i) const
	{
		return static_cast< const float& >( m_pData[i] );
	}

	inline float& operator[](unsigned int i)
	{
		return m_pData[i];
	}

	inline void Scale( const float rhs ) const
	{
		for( unsigned int i = 0; i < m_numDimensions; ++i )
		{
			m_pData[i] *= rhs;
		}
	}

	inline NDPoint& operator=( const NDPoint& rhs )
	{
		for( unsigned int i = 0; i < m_numDimensions; ++i )
		{
			m_pData[i] = rhs.m_pData[i];
		}

		return *this;
	}

protected:

	unsigned int	m_numDimensions;
	float*			m_pData;
};


//---------------------------------------------------------------------------
//! This class generates a random points in a hypercube based on the
//! dimension of the points passed in.
//! This is really a helper class for NDPoints.
//---------------------------------------------------------------------------
class NDPointUniformRandomGenerator
{
public:

	NDPointUniformRandomGenerator( const int randomSeed, const float centerFaceDistance )
		: // Initializer list.
	  	m_randGen( randomSeed ),
		m_centerFaceDistance( centerFaceDistance )
	{
	}
	virtual ~NDPointUniformRandomGenerator()
	{
		;
	}

	inline void GeneratePoint( NDPoint& pointToFill )
	{
		const unsigned int numDimensions = pointToFill.GetNumDimensions();

		for( unsigned int i = 0; i < numDimensions; ++i )
		{
			pointToFill[i] = ( 2.0f * m_centerFaceDistance * m_randGen.Portion() ) -
				m_centerFaceDistance;
		}
	}

protected:

	RandomNumberGenerator	m_randGen;
	float					m_centerFaceDistance;
};


#endif // NDPOINT_HPP