//===========================================================================
//! @file	ndspatialsets.hpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! This is a class to determine if we should keep or throw away a given
//! point.  This is what provides all structure in the space.
//!
//! A spatial set is really just N-dimensional scalar field with an
//! additional acceptance function.
//!
//! It is purposely not templated to allow for use in dynamic applications
//! where the number of different dimesions needed can be driven by real time
//! processes (such as user input or inside the for loop of a test suite).
//===========================================================================
#ifndef NDSPATIALSETS_HPP
#define NDSPATIALSETS_HPP
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ndpoint.hpp"
#include "ndnoise.hpp"
#include "fuzzyselector.hpp"


//---------------------------------------------------------------------------
//! A class to determine if we should keep or throw away a given point.
//! This is what provides all structure in the space.
//---------------------------------------------------------------------------
class NDSpatialSet
{
public:

	NDSpatialSet( const unsigned int numDimensions )
		: // Initializer list.
		m_numDimensions( numDimensions )
	{
	}

	virtual ~NDSpatialSet()
	{
		;
	}

	virtual const bool AcceptMember( const NDPoint& point ) = 0;

protected:

	unsigned int	m_numDimensions;
};


//---------------------------------------------------------------------------
//! This creates an ND shape with fuzzy boundaries.
//! it will tell you if a point is contained in the shape or not.
//---------------------------------------------------------------------------
class NDSpatialSetNoise : public NDSpatialSet
{
public:

	NDSpatialSetNoise(	const unsigned int	numDimensions,
						const int			noiseRandomSeed,
						const float			noiseFrequency,
						const float			acceptanceHardThreshholdPortion,
						const float			acceptanceFuzzyThreshholdPortion,
						const int			acceptanceRandomSeed )
		: // Initializer list.
		NDSpatialSet( numDimensions ),
		m_NDNoise( numDimensions,
			noiseRandomSeed ),
		m_noiseFrequency( noiseFrequency ),
		m_fuzzySelector( acceptanceHardThreshholdPortion,
			acceptanceFuzzyThreshholdPortion,
			acceptanceRandomSeed )
	{
	}

	virtual ~NDSpatialSetNoise()
	{
	}

	virtual const bool AcceptMember( const NDPoint& point )
	{
		// Get the value at this point in the noise field.

		// Scale the point to achieve the same affect as a scaled
		// noise field.
		NDPoint tempPoint( m_numDimensions );
		tempPoint = point;
		tempPoint.Scale( m_noiseFrequency );
		
		const float baseValue = m_NDNoise.GetValue( tempPoint );// -1.0f to 1.0f
		const float value = ( baseValue + 1.0f ) * 0.5f;// 0.0f to 1.0f

		// Detemine if we should accept the point or not.
		return m_fuzzySelector.AcceptValue( value );
	}

protected:

	NDNoise			m_NDNoise;
	float			m_noiseFrequency;
	FuzzySelector	m_fuzzySelector;
};


// DEBUG!! -AC
// You can create your own spatial sets (just N-dimensional scalar field with
// an additional acceptance function). The class immediately below is an
// example of a trivial spatial set that you can use as a basepoint for your
// own.
/*
//---------------------------------------------------------------------------
//! A class to determine if we should keep or throw away a given point.
//! This is what provides all structure in the space.
//---------------------------------------------------------------------------
class NDSpatialSetSphere : public NDSpatialSet
{
public:

	NDSpatialSetSphere( const unsigned int numDimensions, const float radius )
		: // Initializer list.
		NDSpatialSet( numDimensions ),
		m_radius( radius )
	{
	}

	virtual ~NDSpatialSetSphere()
	{

	}

	virtual const bool AcceptMember( const NDPoint& point )
	{
		float distSquared = 0.0f;
		const unsigned int numDimensions = point.GetNumDimensions();

		for( unsigned int i = 0; i < numDimensions; ++i )
		{
			distSquared += ( point[i] * point[i] );
		}

		return ( distSquared < ( m_radius * m_radius ) );
	}

protected:

	float	m_radius;
};
*/
// END DEBUG!!


#endif // NDSPATIALSETS_HPP