//===========================================================================
//! @file	fuzzyselector.hpp
//! @date	08 April 2004
//! @author	John Adam Croston (AC)
//
// Copyright (c) 2004, John Adam Croston.
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
//!
//! This is a class to determine if we should keep or throw away a given
//! value.
//!
//! It is purposely not templated to allow for use in dynamic applications
//! where the number of different dimesions needed can be driven by real time
//! processes (such as user input or inside the for loop of a test suite).
//===========================================================================
#ifndef FUZZYSELECTOR_HPP
#define FUZZYSELECTOR_HPP
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "randgen.hpp"


//---------------------------------------------------------------------------
//! A class to determine if we should keep or throw away a given value.
//---------------------------------------------------------------------------
class FuzzySelector
{
public:

	FuzzySelector( 	const float	acceptanceThreshholdPortion,
					const float	acceptanceFuzzyThreshholdPortion,
					const int	acceptanceRandomSeed )
		: // Initializer list.
		m_acceptanceThreshholdPortion( acceptanceThreshholdPortion ),
		m_acceptanceFuzzyThreshholdPortion( acceptanceFuzzyThreshholdPortion ),
		m_randGen( acceptanceRandomSeed )
	{
	}

	virtual ~FuzzySelector()
	{
		;
	}

	inline const bool AcceptValue( const float valuePortion )
	{
		if( valuePortion < m_acceptanceThreshholdPortion )
		{
			// The value is inside the hard boundary so accept it.
			return true;
		}
		else if( valuePortion < m_acceptanceFuzzyThreshholdPortion ) 
		{
			// The value is between the hard and fuzzy boundaires,
			// so we need to stochasitcallly determine if we should
			// accept the value or not.
			// Values that are closer to the hard boundary should be
			// accepted more often than ones close the the fuzzy
			// boundary.

			// Determine this values chance of acceptance based on
			// where it is between the two boundaries.
			float acceptanceProbability = ( valuePortion - m_acceptanceThreshholdPortion ) /
				( m_acceptanceFuzzyThreshholdPortion - m_acceptanceThreshholdPortion );
			acceptanceProbability = 1.0f - acceptanceProbability;

			// Pick an evenly distributed random number between 0.0 and 1.0.
			// Use this random number and our chance to pick if we
			// accept this value of not.
			if( m_randGen.Portion() <= acceptanceProbability )
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			// The value is outside the fuzzy boundary so reject it.
			return false;
		}
	}

protected:

	float					m_acceptanceThreshholdPortion;
	float					m_acceptanceFuzzyThreshholdPortion;
	RandomNumberGenerator	m_randGen;
};


#endif // FUZZYSELECTOR_HPP

